import logging
import uuid

from sqlalchemy import func, and_
from sqlalchemy.engine.reflection import Inspector
from werkzeug.security import generate_password_hash
from .models import User, Permission, PermissionView, RegisterUser, ViewMenu, Role, Role_Metadata, Company, CompanyPlan, Plans
from ..manager import BaseSecurityManager
from ...models.sqla.interface import SQLAInterface
from ...models.sqla import Base
from ... import const as c
from flask_appbuilder.security.sqla import models as ab_models
from datetime import datetime, timedelta

import requests
import urllib3

urllib3.disable_warnings()
import ast
import json

from flask import g

log = logging.getLogger(__name__)


class SecurityManager(BaseSecurityManager):
    """
        Responsible for authentication, registering security views,
        role and permission auto management

        If you want to change anything just inherit and override, then
        pass your own security manager to AppBuilder.
    """
    user_model = User
    """ Override to set your own User Model """
    role_model = Role
    """ Override to set your own Role Model """

    company_model = Company
    company_plan_model = CompanyPlan
    plans_model = Plans

    permission_model = Permission
    viewmenu_model = ViewMenu
    permissionview_model = PermissionView
    registeruser_model = RegisterUser

    role_model_name = Role_Metadata

    def __init__(self, appbuilder):
        """
            SecurityManager contructor
            param appbuilder:
                F.A.B AppBuilder main object
        """
        super(SecurityManager, self).__init__(appbuilder)
        user_datamodel = SQLAInterface(self.user_model)

        role_model = (self.role_model)
        user_model = self.user_model
        company_model = self.company_model
        company_plan_model = self.company_plan_model
        plans_model = self.plans_model

        if self.auth_type == c.AUTH_DB:
            self.userdbmodelview.datamodel = user_datamodel
            self.userremoteusermodelview.role_model = role_model
            self.userremoteusermodelview.user_model = user_model
            self.userremoteusermodelview.company_model = company_model
            self.userremoteusermodelview.company_plan_model = company_plan_model
            self.userremoteusermodelview.plans_model = plans_model

        elif self.auth_type == c.AUTH_LDAP:
            self.userldapmodelview.datamodel = user_datamodel
        elif self.auth_type == c.AUTH_OID:
            self.useroidmodelview.datamodel = user_datamodel
        elif self.auth_type == c.AUTH_OAUTH:
            self.useroauthmodelview.datamodel = user_datamodel
        elif self.auth_type == c.AUTH_REMOTE_USER:
            self.userremoteusermodelview.datamodel = user_datamodel

        self.userstatschartview.datamodel = user_datamodel
        if self.auth_user_registration:
            self.registerusermodelview.datamodel = SQLAInterface(self.registeruser_model)

        self.rolemodelview.datamodel = SQLAInterface(self.role_model)
        self.permissionmodelview.datamodel = SQLAInterface(self.permission_model)
        self.viewmenumodelview.datamodel = SQLAInterface(self.viewmenu_model)
        self.permissionviewmodelview.datamodel = SQLAInterface(self.permissionview_model)
        self.create_db()

    @property
    def get_session(self):
        return self.appbuilder.get_session

    def register_views(self):
        super(SecurityManager, self).register_views()

    def create_db(self):
        try:
            engine = self.get_session.get_bind(mapper=None, clause=None)
            inspector = Inspector.from_engine(engine)
            if 'ab_user' not in inspector.get_table_names():
                log.info(c.LOGMSG_INF_SEC_NO_DB)
                Base.metadata.create_all(engine)
                log.info(c.LOGMSG_INF_SEC_ADD_DB)
            super(SecurityManager, self).create_db()
        except Exception as e:
            log.error(c.LOGMSG_ERR_SEC_CREATE_DB.format(str(e)))
            exit(1)

    def find_register_user(self, registration_hash):
        return self.get_session.query(self.registeruser_model).filter(
            self.registeruser_model.registration_hash == registration_hash).scalar()

    def add_register_user(self, username, first_name, last_name, email, password='', hashed_password=''):
        """
            Add a registration request for the user.

            :rtype : RegisterUser
        """

        role = 'vip'

        if self.appbuilder.sm.find_role('vip') is None:
            role = 'Alpha'
        else:
            role = 'vip'

        user = self.add_user(username, first_name, last_name, email, self.appbuilder.sm.find_role(role), password)

        return user

    def register_auth_user(self, user_id, access_token, company_name, company_address, username, first_name, last_name, email, password=''):
        """
               Add a registration request for the user.

               :rtype : RegisterUser
           """

        role = 'vip'

        if self.appbuilder.sm.find_role('vip') is None:
            role = 'Alpha'

        user = self.add_user_jungle_auth(user_id, access_token, company_name, company_address, username, first_name, last_name, email,
                                         self.appbuilder.sm.find_role(role), password)

        return user

    def del_register_user(self, register_user):
        """
            Deletes registration object from database

            :param register_user: RegisterUser object to delete
        """
        try:
            self.get_session.delete(register_user)
            self.get_session.commit()
            return True
        except Exception as e:
            log.error(c.LOGMSG_ERR_SEC_DEL_REGISTER_USER.format(str(e)))
            self.get_session.rollback()
            return False

    def find_company_by_name(self, name):
        return self.get_session.query(self.company_model).filter_by(company_name=name).first()

    def find_company_by_id(self, company_id):
        return self.get_session.query(self.company_model).filter_by(id=company_id).first()

    def find_plan_by_company(self, company_id):
        company = self.get_session.query(self.company_model).filter_by(id=company_id).first()
        return company.company_plan

    def insert_payment_history(self, history_row):
        self.get_session.add(history_row)
        self.get_session.commit()

    def insert_payment_log(self, log_row):
        self.get_session.add(log_row)
        self.get_session.commit()

    def get_plan_from_company_id(self, company_id):
        return self.get_session.query(self.company_plan_model).filter_by(company_id=company_id).first()

    def get_plan_from_id(self, plan_id):
        return self.get_session.query(self.plans_model).filter_by(id=plan_id).first()

    def get_base_price_from_plan(self, plan_id):
        return self.get_session.query(self.plans_model).filter_by(id=plan_id).first()

    def update_company_plan(self, company_id, plan_id):
        company_plan = self.get_session.query(self.company_plan_model).filter_by(company_id=company_id).first()
        setattr(company_plan, 'plan_id', plan_id)
        self.get_session.commit()

    def get_company_users(self, company_name=None, company_id=None):

        users = None

        if company_name:
            if self.get_session.query(self.company_model).filter_by(company_name=company_name).first() is None:
                return None
            else:
                users = self.get_session.query(self.company_model).filter_by(company_name=company_name).first().users

        elif company_id:
            if self.get_session.query(self.company_model).filter_by(id=company_id).first() is None:
                return None
            else:
                users = self.get_session.query(self.company_model).filter_by(id=company_id).first().users

        return users

    def find_user(self, username=None, email=None):
        """
            Finds user by username or email
        """
        # print("in find_user login")
        if username:
            return self.get_session.query(self.user_model).filter(func.lower(self.user_model.username) == func.lower(username)).first()
        elif email:
            return self.get_session.query(self.user_model).filter_by(email=email).first()
        # return self.get_session.query(self.user_model).filter_by(email=email).first()

    def find_user_by_id(self, user_id):
        return self.get_session.query(self.user_model).filter_by(auth_id=user_id).first()

    def find_user_by_company(self, company_name):
        return self.get_session.query(self.user_model).filter_by(company_name=company_name).first()

    def find_user_by_name(self, name):
        return self.get_session.query(self.user_model).filter_by(name=name).first()

    def get_all_users(self):
        return self.get_session.query(self.user_model).all()

    def add_user(self, username, first_name, last_name, email, role, password='', hashed_password=''):
        """
            Generic function to create user
        """
        company_name = 'Demo Company'

        role_name = str(company_name).lower().strip().replace(".", "").replace(",", "").replace("-", "").replace(" ", "_") + "_adm"
        self.add_company_admin_perm(role_name)

        try:
            user = self.user_model()
            user.first_name = first_name
            user.last_name = last_name
            user.username = username
            user.email = email
            user.active = True
            user.roles.append(self.appbuilder.sm.find_role(role_name))
            user.onboarding = 3

            user.auth_id = -1
            user.access_token = 'demo_access'
            user.company_name = company_name
            user.company_address = 'Demo Address'
            user.user_type = 0
            user.user_parent = 0
            user.company_id = -1

            if hashed_password:
                user.password = hashed_password
            else:
                user.password = generate_password_hash(password)
            self.get_session.add(user)
            self.get_session.commit()
            log.info(c.LOGMSG_INF_SEC_ADD_USER.format(username))
            return user
        except Exception as e:
            log.error(c.LOGMSG_ERR_SEC_ADD_USER.format(str(e)))
            return False

    def add_user_jungle_auth(self, user_id, access_token, company_name, company_address, username, first_name, last_name, email, role,
                             password=''):
        """
            Generic function to create user
        """
        role_name = str(company_name).lower().strip().replace(".", "").replace(",", "").replace("-", "").replace(" ", "_") + "_adm"

        self.add_company_admin_perm(role_name)

        try:
            company = self.company_model()
            company_plan = self.company_plan_model()
            user = self.user_model()

            if self.find_company_by_name(company_name):
                company = self.find_company_by_name(company_name)
                company.company_plan = self.find_plan_by_company(company.id)
                company_user_id = (self.find_user_by_id(company.admin_id)).id
                user.user_type = 2
                user.user_parent = company_user_id
                plan = self.get_plan_from_id(company.company_plan.plan_id)
                if len(self.get_company_users(company_name=company_name)) >= plan.base_users:
                    user.is_extra = 1
            else:
                company.company_name = company_name
                company.company_address = company_address
                company_plan.plan_id = 1
                company_plan.start_date = datetime.now()
                company_plan.end_date = company_plan.start_date + timedelta(days=14)
                company.company_plan = company_plan
                user.user_type = 1
                user.user_parent = -1
                company.admin_id = user_id
                company.company_plan = company_plan

            user.first_name = first_name
            user.last_name = last_name
            user.username = username
            user.email = email
            user.active = True
            user.roles.append(self.appbuilder.sm.find_role(role_name))
            user.password = generate_password_hash(password)
            user.onboarding = 0
            user.auth_id = user_id
            user.access_token = access_token

            company.users.append(user)

            self.get_session.add(company)
            self.get_session.commit()
            return user
        except Exception as e:
            log.error(c.LOGMSG_ERR_SEC_ADD_USER.format(str(e)))
            return False

    def count_users(self):
        return self.get_session.query(func.count('*')).select_from(self.user_model).scalar()

    def update_user(self, user):
        try:
            self.get_session.merge(user)
            self.get_session.commit()
            log.info(c.LOGMSG_INF_SEC_UPD_USER.format(user))
        except Exception as e:
            log.error(c.LOGMSG_ERR_SEC_UPD_USER.format(str(e)))
            self.get_session.rollback()
            return False

    def get_user_by_id(self, pk):
        return self.get_session.query(self.user_model).get(pk)

    """
    -----------------------
     PERMISSION MANAGEMENT
    -----------------------
    """

    def add_role(self, name):
        role = self.find_role(name)

        if role is None:
            try:
                role = self.role_model()
                role.name = name
                self.get_session.add(role)
                self.get_session.commit()
                log.info(c.LOGMSG_INF_SEC_ADD_ROLE.format(name))
                return role
            except Exception as e:
                log.error(c.LOGMSG_ERR_SEC_ADD_ROLE.format(str(e)))
                self.get_session.rollback()
        return role

    def find_role(self, name):
        return self.get_session.query(self.role_model).filter_by(name=name).first()

    def get_all_roles(self):
        return self.get_session.query(self.role_model).all()

    def get_public_permissions(self):
        role = self.get_session.query(self.role_model).filter_by(name=self.auth_role_public).first()
        return role.permissions

    def find_permission(self, name):
        """
            Finds and returns a Permission by name
        """
        return self.get_session.query(self.permission_model).filter_by(name=name).first()

    def add_permission(self, name):
        """
            Adds a permission to the backend, model permission

            :param name:
                name of the permission: 'can_add','can_edit' etc...
        """
        perm = self.find_permission(name)
        if perm is None:
            try:
                perm = self.permission_model()
                perm.name = name
                self.get_session.add(perm)
                self.get_session.commit()
                return perm
            except Exception as e:
                log.error(c.LOGMSG_ERR_SEC_ADD_PERMISSION.format(str(e)))
                self.get_session.rollback()
        return perm

    def del_permission(self, name):
        """
            Deletes a permission from the backend, model permission

            :param name:
                name of the permission: 'can_add','can_edit' etc...
        """
        perm = self.find_permission(name)
        if perm:
            try:
                self.get_session.delete(perm)
                self.get_session.commit()
            except Exception as e:
                log.error(c.LOGMSG_ERR_SEC_DEL_PERMISSION.format(str(e)))
                self.get_session.rollback()

    '''Custom methods'''

    def add_role_name(self, name):
        role_name = self.find_role_metadata(name)
        if role_name is None:
            try:
                role_name = self.role_model_name()
                role_name.name = name
                self.get_session.add(role_name)
                self.get_session.commit()
                log.info(c.LOGMSG_INF_SEC_ADD_ROLE.format(name))
                return role_name
            except Exception as e:
                log.error(c.LOGMSG_ERR_SEC_ADD_ROLE.format(str(e)))
                self.get_session.rollback()
        return role_name

    def find_role_metadata(self, name):
        return self.get_session.query(self.role_model_name).filter_by(name=name).first()

    """
    ----------------------
     PRIMITIVES VIEW MENU
    ----------------------
    """

    def find_view_menu(self, name):
        """
            Finds and returns a ViewMenu by name
        """
        return self.get_session.query(self.viewmenu_model).filter_by(name=name).first()

    def get_all_view_menu(self):
        return self.get_session.query(self.viewmenu_model).all()

    def add_view_menu(self, name):
        """
            Adds a view or menu to the backend, model view_menu
            param name:
                name of the view menu to add
        """
        view_menu = self.find_view_menu(name)
        if view_menu is None:
            try:
                view_menu = self.viewmenu_model()
                view_menu.name = name
                self.get_session.add(view_menu)
                self.get_session.commit()
                return view_menu
            except Exception as e:
                log.error(c.LOGMSG_ERR_SEC_ADD_VIEWMENU.format(str(e)))
                self.get_session.rollback()
        return view_menu

    def del_view_menu(self, name):
        """
            Deletes a ViewMenu from the backend

            :param name:
                name of the ViewMenu
        """
        obj = self.find_view_menu(name)
        if obj:
            try:
                self.get_session.delete(obj)
                self.get_session.commit()
            except Exception as e:
                log.error(c.LOGMSG_ERR_SEC_DEL_PERMISSION.format(str(e)))
                self.get_session.rollback()

    """
    ----------------------
     PERMISSION VIEW MENU
    ----------------------
    """

    def find_permission_view_menu(self, permission_name, view_menu_name):
        """
            Finds and returns a PermissionView by names
        """
        permission = self.find_permission(permission_name)
        view_menu = self.find_view_menu(view_menu_name)
        return self.get_session.query(self.permissionview_model).filter_by(permission=permission, view_menu=view_menu).first()

    def find_permissions_view_menu(self, view_menu):
        """
            Finds all permissions from ViewMenu, returns list of PermissionView

            :param view_menu: ViewMenu object
            :return: list of PermissionView objects
        """
        return self.get_session.query(self.permissionview_model).filter_by(view_menu_id=view_menu.id).all()

    def add_permission_view_menu(self, permission_name, view_menu_name):
        """
            Adds a permission on a view or menu to the backend

            :param permission_name:
                name of the permission to add: 'can_add','can_edit' etc...
            :param view_menu_name:
                name of the view menu to add
        """
        vm = self.add_view_menu(view_menu_name)
        perm = self.add_permission(permission_name)
        pv = self.permissionview_model()
        pv.view_menu_id, pv.permission_id = vm.id, perm.id
        try:
            self.get_session.add(pv)
            self.get_session.commit()
            log.info(c.LOGMSG_INF_SEC_ADD_PERMVIEW.format(str(pv)))
            return pv
        except Exception as e:
            log.error(c.LOGMSG_ERR_SEC_ADD_PERMVIEW.format(str(e)))
            self.get_session.rollback()

    def del_permission_view_menu(self, permission_name, view_menu_name):
        try:
            pv = self.find_permission_view_menu(permission_name, view_menu_name)
            # delete permission on view
            self.get_session.delete(pv)
            self.get_session.commit()
            # if no more permission on permission view, delete permission
            pv = self.get_session.query(self.permissionview_model).filter_by(permission=pv.permission).all()
            if not pv:
                self.del_permission(pv.permission.name)
            log.info(c.LOGMSG_INF_SEC_DEL_PERMVIEW.format(permission_name, view_menu_name))
        except Exception as e:
            log.error(c.LOGMSG_ERR_SEC_DEL_PERMVIEW.format(str(e)))
            self.get_session.rollback()

    def exist_permission_on_views(self, lst, item):
        for i in lst:
            if i.permission and i.permission.name == item:
                return True
        return False

    def exist_permission_on_view(self, lst, permission, view_menu):
        for i in lst:
            if i.permission.name == permission and i.view_menu.name == view_menu:
                return True
        return False

    def add_permission_role(self, role, perm_view):
        """
            Add permission-ViewMenu object to Role

            :param role:
                The role object
            :param perm_view:
                The PermissionViewMenu object
        """
        if perm_view not in role.permissions:
            try:
                role.permissions.append(perm_view)
                self.get_session.merge(role)
                self.get_session.commit()
                log.info(c.LOGMSG_INF_SEC_ADD_PERMROLE.format(str(perm_view), role.name))
            except Exception as e:
                log.error(c.LOGMSG_ERR_SEC_ADD_PERMROLE.format(str(e)))
                self.get_session.rollback()

    def del_permission_role(self, role, perm_view):
        """
            Remove permission-ViewMenu object to Role

            :param role:
                The role object
            :param perm_view:
                The PermissionViewMenu object
        """
        if perm_view in role.permissions:
            try:
                role.permissions.remove(perm_view)
                self.get_session.merge(role)
                self.get_session.commit()
                log.info(c.LOGMSG_INF_SEC_DEL_PERMROLE.format(str(perm_view), role.name))
            except Exception as e:
                log.error(c.LOGMSG_ERR_SEC_DEL_PERMROLE.format(str(e)))
                self.get_session.rollback()

    def add_company_admin_perm(self, role_name):

        sesh = self.get_session()
        pvms = sesh.query(ab_models.PermissionView).all()
        pvms = [p for p in pvms if p.permission and p.view_menu]
        role = self.add_role(role_name)
        role_pvms = [p for p in pvms if self.is_alpha_pvm(p) and 'delete on RoleModelView' not in str(p) and 'edit on RoleModelView' not in str(p)]
        role.permissions = role_pvms
        sesh.merge(role)
        sesh.commit()

    def is_user_defined_permission(self, perm):
        return perm.permission.name in [
            'database_access',
            'schema_access',
            'datasource_access',
            'metric_access',
        ]

    def is_admin_only(self, pvm):
        # not readonly operations on read only model views allowed only for admins
        if (pvm.view_menu.name in {
            'DruidClusterModelView',
            # 'ResetMyPasswordView',
        } and
                pvm.permission.name not in {
                    'can_show',
                    'can_list',
                }):
            return True
        return (pvm.view_menu.name in {
            'AccessRequestsModelView',
            'Refresh Druid Metadata',
            'ResetPasswordView',
        } or
                pvm.permission.name in {
                    'can_override_role_permissions',
                    'can_sync_druid_source',
                    'can_override_role_permissions',
                    'can_approve',
                    'can_update_role',
                })

    def is_alpha_pvm(self, pvm):
        return not (self.is_user_defined_permission(pvm) or self.is_admin_only(pvm))
