import datetime
from flask import g
from sqlalchemy import Table, Column, Integer, String, Boolean, DateTime, ForeignKey, Sequence
from sqlalchemy.orm import relationship, backref
from sqlalchemy.ext.declarative import declared_attr
from ... import Model
from ..._compat import as_unicode

_dont_audit = False


class Permission(Model):
    __tablename__ = 'ab_permission'
    id = Column(Integer, Sequence('ab_permission_id_seq'), primary_key=True)
    name = Column(String(100), unique=True, nullable=False)

    def __repr__(self):
        return self.name


class ViewMenu(Model):
    __tablename__ = 'ab_view_menu'
    id = Column(Integer, Sequence('ab_view_menu_id_seq'), primary_key=True)
    name = Column(String(100), unique=True, nullable=False)

    def __eq__(self, other):
        return (isinstance(other, self.__class__)) and (self.name == other.name)

    def __neq__(self, other):
        return self.name != other.name

    def __repr__(self):
        return self.name


class PermissionView(Model):
    __tablename__ = 'ab_permission_view'
    id = Column(Integer, Sequence('ab_permission_view_id_seq'), primary_key=True)
    permission_id = Column(Integer, ForeignKey('ab_permission.id'))
    permission = relationship("Permission")
    view_menu_id = Column(Integer, ForeignKey('ab_view_menu.id'))
    view_menu = relationship("ViewMenu")

    def __repr__(self):
        return str(self.permission).replace('_', ' ') + ' on ' + str(self.view_menu)


assoc_permissionview_role = Table('ab_permission_view_role', Model.metadata,
                                  Column('id', Integer, Sequence('ab_permission_view_role_id_seq'), primary_key=True),
                                  Column('permission_view_id', Integer, ForeignKey('ab_permission_view.id')),
                                  Column('role_id', Integer, ForeignKey('ab_role.id'))
                                  )

assoc_roles_visibility = Table('ab_roles_assoc', Model.metadata,
                               Column('id', Integer, primary_key=True),
                               Column('role_id', Integer, ForeignKey('ab_role.id')),
                               Column('role_metadata_id', Integer, ForeignKey('ab_role_metadata.id'))
                               )


class Role(Model):
    __tablename__ = 'ab_role'

    id = Column(Integer, Sequence('ab_role_id_seq'), primary_key=True)
    name = Column(String(64), unique=True, nullable=False)
    permissions = relationship('PermissionView', secondary=assoc_permissionview_role, backref='role')
    roles_visible = relationship('Role_Metadata', secondary=assoc_roles_visibility);

    def __repr__(self):
        return self.name


class Role_Metadata(Model):
    __tablename__ = 'ab_role_metadata'

    id = Column(Integer, primary_key=True)
    name = Column(String(64), unique=True, nullable=False)

    def __repr__(self):
        return self.name


assoc_user_role = Table('ab_user_role', Model.metadata,
                        Column('id', Integer, Sequence('ab_user_role_id_seq'), primary_key=True),
                        Column('user_id', Integer, ForeignKey('ab_user.id')),
                        Column('role_id', Integer, ForeignKey('ab_role.id'))
                        )


class User(Model):
    __tablename__ = 'ab_user'
    id = Column(Integer, Sequence('ab_user_id_seq'), primary_key=True)
    first_name = Column(String(64), nullable=False)
    last_name = Column(String(64), nullable=True)
    username = Column(String(64), nullable=True, default='user')
    password = Column(String(256))
    active = Column(Boolean)
    email = Column(String(64), unique=True, nullable=False)
    auth_id = Column(Integer, comment='Jungle Auth User ID')
    access_token = Column(String(256))
    onboarding = Column(Integer, default=0, comment='0->incomplete, 1->partial, 2->complete')
    last_login = Column(DateTime)
    login_count = Column(Integer)
    fail_login_count = Column(Integer)
    is_extra = Column(Integer, default=0)
    roles = relationship('Role', secondary=assoc_user_role, backref='user')
    last_payment_on = Column(DateTime, default=datetime.datetime.now, nullable=True)
    created_on = Column(DateTime, default=datetime.datetime.now, nullable=True)
    changed_on = Column(DateTime, default=datetime.datetime.now, nullable=True)
    user_parent = Column(String(64), nullable=False)
    user_type = Column(Integer, nullable=False, comment='0->super user, 1->admin, 2->basic')
    company_id = Column(Integer, ForeignKey('tb_company.id'))

    @declared_attr
    def created_by_fk(self):
        return Column(Integer, ForeignKey('ab_user.id'),
                      default=self.get_user_id, nullable=True)

    @declared_attr
    def changed_by_fk(self):
        return Column(Integer, ForeignKey('ab_user.id'),
                      default=self.get_user_id, nullable=True)

    created_by = relationship("User", backref=backref("created", uselist=True),
                              remote_side=[id], primaryjoin='User.created_by_fk == User.id', uselist=False)
    changed_by = relationship("User", backref=backref("changed", uselist=True),
                              remote_side=[id], primaryjoin='User.changed_by_fk == User.id', uselist=False)

    @classmethod
    def get_user_id(cls):
        try:
            return g.user.id
        except Exception as e:
            return None

    def is_authenticated(self):
        return True

    def is_active(self):
        return self.active

    def is_anonymous(self):
        return False

    def get_id(self):
        return as_unicode(self.id)

    def get_full_name(self):
        return u'{0} {1}'.format(self.first_name, self.last_name)

    def __repr__(self):
        return self.get_full_name()

    def get_email(self):
        return u'{0}'.format(self.email)

    def get_actual_user_id(self):
        return u'{0}'.format(self.auth_id)

    def get_access_token(self):
        return u'{0}'.format(self.access_token)

    def get_company_name(self):
        return u'{0}'.format(self.company_name)

    def get_company_address(self):
        return u'{0}'.format(self.company_address)


class RegisterUser(Model):
    __tablename__ = 'ab_register_user'
    id = Column(Integer, Sequence('ab_register_user_id_seq'), primary_key=True)
    first_name = Column(String(64), nullable=False)
    last_name = Column(String(64), nullable=False)
    username = Column(String(64), unique=True, nullable=False)
    password = Column(String(256))
    email = Column(String(64), nullable=False)
    registration_date = Column(DateTime, default=datetime.datetime.now, nullable=True)
    registration_hash = Column(String(256))


class Company(Model):
    __tablename__ = 'tb_company'
    id = Column(Integer, Sequence('tb_company_id_seq'), primary_key=True)
    company_name = Column(String(128), nullable=True, unique=True)
    company_address = Column(String(256))
    payment_status = Column(Integer, default=0)
    admin_id = Column(Integer, default=-1, comment='Company Admin Auth ID')
    users = relationship('User', backref='company')
    company_plan = relationship('CompanyPlan', uselist=False, backref='company')


class CompanyPlan(Model):
    __tablename__ = 'tb_company_plan'
    id = Column(Integer, Sequence('tb_company_plan_id_seq'), primary_key=True)
    company_id = Column(Integer, ForeignKey('tb_company.id'))
    plan_id = Column(Integer, ForeignKey('tb_plans.id'))
    start_date = Column(DateTime)
    end_date = Column(DateTime)
    billing_cycle = Column(Integer, default=1, comment='1->Monthly, 2->Annually')
    outstanding_amount = Column(Integer, default=0)
    credit = Column(Integer, default=0)
    discounted_amount = Column(Integer, default=0)
    is_active = Column(Integer, default=1)
    created_on = Column(DateTime, default=datetime.datetime.now, nullable=True)


class Plans(Model):
    __tablename__ = 'tb_plans'
    id = Column(Integer, Sequence('tb_plans_id_seq'), primary_key=True)
    plan_name = Column(String(64), unique=True)
    base_price = Column(Integer, default=0)
    variable_price = Column(Integer, default=0)
    fixed_price = Column(Integer, default=0, comment='Monthly')
    plan_period = Column(Integer, comment='In Days')
    trial_period = Column(Integer, comment='In Days')
    base_users = Column(Integer, default=1)
    number_of_dashboards = Column(Integer, default=1)
    number_of_kpis = Column(Integer, default=10)


class PaymentHistory(Model):
    __tablename__ = 'tb_payment_history'
    id = Column(Integer, Sequence('tb_payment_history_id_seq'), primary_key=True)
    payment_mode = Column(String(64))
    auth_id = Column(Integer)
    company_id = Column(Integer)
    billing_plan_id = Column(Integer)
    billing_amount = Column(Integer)
    status = Column(Integer)
    billed_on = Column(DateTime, default=datetime.datetime.now, nullable=True)
    transaction_status = Column(Integer)
    transaction_id = Column(String(64))


class PaymentLogs(Model):
    __tablename__ = 'tb_payment_logs'
    id = Column(Integer, Sequence('tb_payment_logs_id_seq'), primary_key=True)
    initiator_id = Column(Integer, ForeignKey('ab_user.id'), nullable=False)
    initiator_auth_id = Column(Integer, nullable=False)
    company_id = Column(Integer, ForeignKey('tb_company.id'), nullable=False)
    billing_amount = Column(Integer, nullable=False)
    selected_plan = Column(Integer, nullable=False)
    status = Column(Integer, nullable=False, comment='-1->Initial, 0->Unsuccessful, 1->Successful')
    description = Column(String(256), nullable=True)
    initiated_on = Column(DateTime, default=datetime.datetime.now, nullable=True)

