import re
import datetime
import logging
from flask import flash, redirect, session, url_for, request, g, make_response, jsonify, abort
from werkzeug.security import generate_password_hash
from wtforms import validators, PasswordField
from wtforms.validators import EqualTo
from flask_babel import lazy_gettext
from flask_login import login_user, logout_user

from ..views import ModelView, SimpleFormView, expose
from ..baseviews import BaseView
from ..charts.views import DirectByChartView
from ..fieldwidgets import BS3PasswordFieldWidget
from ..actions import action
from .._compat import as_unicode
from .forms import LoginForm_db, LoginForm_oid, ResetPasswordForm, UserInfoEdit
from .decorators import has_access
import requests
from kato.credentials import BASE_API_URL, KATO_OFFERING_ID, JUNGLE_AUTH_KEY, KATO_BASE_URL
from colorama import Fore
import json

from ..urltools import *
import requests
import json
from sqlalchemy import func, and_, or_
from flask_appbuilder.models.sqla.filters import FilterEqualFunction
from flask_appbuilder.models.sqla.filters import BaseFilter
from ..security.sqla.models import Role, Role_Metadata, assoc_roles_visibility, User, PaymentHistory, PaymentLogs
from flask_sqlalchemy import SQLAlchemy
import hashlib

log = logging.getLogger(__name__)


class PermissionModelView(ModelView):
    route_base = '/permissions'
    base_permissions = ['can_list']

    list_title = lazy_gettext('List Base Permissions')
    show_title = lazy_gettext('Show Base Permission')
    add_title = lazy_gettext('Add Base Permission')
    edit_title = lazy_gettext('Edit Base Permission')

    label_columns = {'name': lazy_gettext('Name')}


class ViewMenuModelView(ModelView):
    route_base = '/viewmenus'
    base_permissions = ['can_list']

    list_title = lazy_gettext('List View Menus')
    show_title = lazy_gettext('Show View Menu')
    add_title = lazy_gettext('Add View Menu')
    edit_title = lazy_gettext('Edit View Menu')

    label_columns = {'name': lazy_gettext('Name')}


class PermissionViewModelView(ModelView):
    route_base = '/permissionviews'
    base_permissions = ['can_list']

    list_title = lazy_gettext('List Permissions on Views/Menus')
    show_title = lazy_gettext('Show Permission on Views/Menus')
    add_title = lazy_gettext('Add Permission on Views/Menus')
    edit_title = lazy_gettext('Edit Permission on Views/Menus')

    label_columns = {'permission': lazy_gettext('Permission'), 'view_menu': lazy_gettext('View/Menu')}
    list_columns = ['permission', 'view_menu']


class ResetMyPasswordView(SimpleFormView):
    """
        View for resetting own user password
    """
    route_base = '/resetmypassword'
    form = ResetPasswordForm
    form_title = lazy_gettext('Reset Password Form')
    redirect_url = '/'
    message = lazy_gettext('Password Changed')
    error = lazy_gettext('There was an error in updating the password.')

    def form_post(self, form):

        if 'Reset Password' in self.form_title:
            if not self.update_password_jungle_auth(request.form, g.user.auth_id):
                flash(as_unicode(self.error), 'danger')
                raise Exception("Error")

            self.appbuilder.sm.reset_password(g.user.id, form.password.data)
            flash(as_unicode(self.message), 'info')

    def update_password_jungle_auth(self, post_data, user_id):

        request_data = {
            "auth_key": JUNGLE_AUTH_KEY,
            "user_id": user_id,
            "offering": KATO_OFFERING_ID,
            "updates":
                {
                    "password": hashlib.md5(post_data['password'].encode('utf8')).hexdigest()
                }
        }

        # print("Password = {}    Hex = {}".format(post_data['password'], hashlib.md5(post_data['password'].encode('utf8')).hexdigest()))

        api_response = requests.post(BASE_API_URL + '/update_user_detail', json=request_data)
        api_data = api_response.json()

        # print("Update response = {}".format(api_data))

        if api_data['status'] == 200:
            return True

        return False


class ResetPasswordView(SimpleFormView):
    """
        View for reseting all users password
    """
    route_base = '/resetpassword'
    form = ResetPasswordForm
    form_title = lazy_gettext('Reset Password Form')
    redirect_url = '/'
    message = lazy_gettext('Password Changed')

    def form_post(self, form):
        pk = request.args.get('pk')
        self.appbuilder.sm.reset_password(pk, form.password.data)
        flash(as_unicode(self.message), 'info')


class UserInfoEditView(SimpleFormView):
    form = UserInfoEdit
    form_title = lazy_gettext('Edit User Information')
    redirect_url = '/'
    message = lazy_gettext('User information changed')

    def form_get(self, form):
        item = self.appbuilder.sm.get_user_by_id(g.user.id)
        # fills the form generic solution
        for key, value in form.data.items():
            if key == 'csrf_token': continue
            form_field = getattr(form, key)
            form_field.data = getattr(item, key)

    def form_post(self, form):
        form = self.form.refresh(request.form)
        item = self.appbuilder.sm.get_user_by_id(g.user.id)
        form.populate_obj(item)
        self.appbuilder.sm.update_user(item)
        flash(as_unicode(self.message), 'info')


class UserModelView(ModelView):
    route_base = '/users'

    list_title = lazy_gettext('List Users')
    show_title = lazy_gettext('Show User')
    add_title = lazy_gettext('Add User')
    edit_title = lazy_gettext('Edit User')

    label_columns = {'get_full_name': lazy_gettext('Full Name'),
                     'first_name': lazy_gettext('First Name'),
                     'last_name': lazy_gettext('Last Name'),
                     'username': lazy_gettext('User Name'),
                     'password': lazy_gettext('Password'),
                     'active': lazy_gettext('Is Active?'),
                     'email': lazy_gettext('EMail'),
                     'roles': lazy_gettext('Role'),
                     'last_login': lazy_gettext('Last login'),
                     'login_count': lazy_gettext('Login count'),
                     'fail_login_count': lazy_gettext('Failed login count'),
                     'created_on': lazy_gettext('Created on'),
                     'created_by': lazy_gettext('Created by'),
                     'changed_on': lazy_gettext('Changed on'),
                     'changed_by': lazy_gettext('Changed by'),
                     'user_type': lazy_gettext('User Type'),
                     'name': lazy_gettext('User Name')}

    description_columns = {'first_name': lazy_gettext('Write the user first name or names'),
                           'last_name': lazy_gettext('Write the user last name'),
                           'username': lazy_gettext(
                               'Username valid for authentication on DB or LDAP, unused for OID auth'),
                           'password': lazy_gettext(
                               'Please use a good password policy, this application does not check this for you'),
                           'active': lazy_gettext('It\'s not a good policy to remove a user, just make it inactive'),
                           'email': lazy_gettext('The user\'s email, this will also be used for OID auth'),
                           'roles': lazy_gettext(
                               'The user role on the application, this will associate with a list of permissions'),
                           'conf_password': lazy_gettext('Please rewrite the user\'s password to confirm')}

    list_columns = ['first_name', 'last_name', 'email', 'active']

    show_fieldsets = [
        (lazy_gettext('Personal Info'),
         {'fields': ['first_name', 'last_name', 'email', 'roles'], 'expanded': True}),
        (lazy_gettext('User info'),
         {'fields': ['active', 'login_count']}),
    ]

    user_show_fieldsets = [
        (lazy_gettext('Personal Info'),
         {'fields': ['first_name', 'last_name', 'email'], 'expanded': True}),
        (lazy_gettext('User info'),
         {'fields': ['active', 'login_count']})
    ]

    search_exclude_columns = ['password']

    add_columns = ['first_name', 'last_name', 'active', 'email', 'roles', 'password', 'conf_password']
    edit_columns = ['first_name', 'last_name', 'active', 'email']
    # edit_columns = ['first_name', 'last_name', 'username', 'active', 'email', 'roles']
    user_info_title = lazy_gettext("Your user information")

    @expose('/userinfo/')
    @has_access
    def userinfo(self):
        item = self.datamodel.get(g.user.id, self._base_filters)
        widgets = self._get_show_widget(g.user.id, item, show_fieldsets=self.user_show_fieldsets)
        self.update_redirect()
        return self.render_template(self.show_template,
                                    title=self.user_info_title,
                                    widgets=widgets,
                                    appbuilder=self.appbuilder)

    @action('userinfoedit', lazy_gettext("Edit User"), "", "fa-edit", multiple=False)
    def userinfoedit(self, item):
        return redirect(url_for(self.appbuilder.sm.userinfoeditview.__name__ + '.this_form_get'))

    def _subclass_container(self):
        _parent_class = self  # Create access to parent class.

        class inner_filter(BaseFilter):
            def __int__(self):
                self._parent_class = _parent_class

            def get_session(self):
                return _parent_class.appbuilder.get_session

            def apply(self, query, value):
                """
                    Override this to implement your own new filters
                """

                if g.user.user_type == 0:
                    return query
                elif g.user.user_type == 1:
                    return query.filter(User.company_id == g.user.company_id)
                else:
                    return query.filter(or_(User.id == g.user.id, User.user_parent == g.user.id))

        return {"inner_filter": inner_filter}

    def _get_list_widget(self, filters,
                         actions=None,
                         order_column='',
                         order_direction='',
                         page=None,
                         page_size=None,
                         widgets=None,
                         **args):

        """ get joined base filter and current active filter for query """
        widgets = widgets or {}
        actions = actions or self.actions
        page_size = page_size or self.page_size

        if not order_column and self.base_order:
            order_column, order_direction = self.base_order

        # TODO : Visible Redundancy at some places ; Will resolve in upcoming releases. Shortage of time is forcing to continue;

        filter_custom = [["custom_filter", self._subclass_container()['inner_filter'], lambda: []]]
        customized_filters = self.datamodel.get_filters().add_filter_list(filter_custom)
        joined_filters = filters.get_joined_filters(self._base_filters)
        joined_filters = joined_filters.get_joined_filters(customized_filters)

        count, lst = self.datamodel.query(joined_filters, order_column, order_direction, page=page, page_size=page_size)
        pks = self.datamodel.get_keys(lst)

        widgets['list'] = self.list_widget(label_columns=self.label_columns,
                                           include_columns=self.list_columns,
                                           value_columns=self.datamodel.get_values(lst, self.list_columns),
                                           order_columns=self.order_columns,
                                           formatters_columns=self.formatters_columns,
                                           page=page,
                                           page_size=page_size,
                                           count=count,
                                           pks=pks,
                                           actions=actions,
                                           filters=filters,
                                           modelview_name=self.__class__.__name__)

        return widgets;


class UserOIDModelView(UserModelView):
    """
        View that add OID specifics to User view.
        Override to implement your own custom view.
        Then override useroidmodelview property on SecurityManager
    """
    pass


class UserLDAPModelView(UserModelView):
    """
        View that add LDAP specifics to User view.
        Override to implement your own custom view.
        Then override userldapmodelview property on SecurityManager
    """
    pass


class UserOAuthModelView(UserModelView):
    """
        View that add OAUTH specifics to User view.
        Override to implement your own custom view.
        Then override userldapmodelview property on SecurityManager
    """
    pass


class UserRemoteUserModelView(UserModelView):
    """
        View that add REMOTE_USER specifics to User view.
        Override to implement your own custom view.
        Then override userldapmodelview property on SecurityManager
    """
    pass


'''Custom Class'''

from flask_appbuilder.widgets import RenderTemplateWidget


class FormUserWidget(RenderTemplateWidget):
    """
            Custom Template for linking Scopeworker Users with KATO

            List User Widget implements a Template as an widget.
            It takes the following arguments

            label_columns = []
            include_columns = []
            value_columns = []
            order_columns = []
            page = None
            page_size = None
            count = 0
            pks = []
            actions = None
            filters = {}
            modelview_name = 'USER'
        """
    template = 'appbuilder/general/widgets/form_add_user.html'


class UserDBModelView(UserModelView):
    """
        View that add DB specifics to User view.
        Override to implement your own custom view.
        Then override userdbmodelview property on SecurityManager
    """
    add_form_extra_fields = {'password': PasswordField(lazy_gettext('Password'),
                                                       description=lazy_gettext(
                                                           'Please use a good password policy, this application does not check this for you'),
                                                       validators=[validators.DataRequired()],
                                                       widget=BS3PasswordFieldWidget()),
                             'conf_password': PasswordField(lazy_gettext('Confirm Password'),
                                                            description=lazy_gettext(
                                                                'Please rewrite the user\'s password to confirm'),
                                                            validators=[EqualTo('password', message=lazy_gettext(
                                                                'Passwords must match'))],
                                                            widget=BS3PasswordFieldWidget())}

    # add_columns = ['first_name', 'last_name', 'active', 'email', 'password', 'conf_password']
    add_columns = ['first_name', 'last_name', 'active', 'email', 'password', 'conf_password']

    @expose('/show/<pk>', methods=['GET'])
    @has_access
    def show(self, pk):
        actions = {}
        actions['resetpasswords'] = self.actions.get('resetpasswords')
        item = self.datamodel.get(pk, self._base_filters)
        if not item:
            abort(404)
        widgets = self._get_show_widget(pk, item, actions=actions)
        self.update_redirect()
        return self.render_template(self.show_template,
                                    pk=pk,
                                    title=self.show_title,
                                    widgets=widgets,
                                    appbuilder=self.appbuilder,
                                    related_views=self._related_views)

    @expose('/userinfo/')
    @has_access
    def userinfo(self):
        actions = {}
        actions['resetmypassword'] = self.actions.get('resetmypassword')
        actions['userinfoedit'] = self.actions.get('userinfoedit')

        item = self.datamodel.get(g.user.id, self._base_filters)
        widgets = self._get_show_widget(g.user.id, item, actions=actions, show_fieldsets=self.user_show_fieldsets)
        self.update_redirect()
        return self.render_template(self.show_template,
                                    title=self.user_info_title,
                                    widgets=widgets,
                                    appbuilder=self.appbuilder,
                                    )

    @action('resetmypassword', lazy_gettext("Reset my password"), "", "fa-lock", multiple=False)
    def resetmypassword(self, item):
        return redirect(url_for(self.appbuilder.sm.resetmypasswordview.__name__ + '.this_form_get'))

    @action('resetpasswords', lazy_gettext("Reset Password"), "", "fa-lock", multiple=False)
    def resetpasswords(self, item):
        return redirect(url_for(self.appbuilder.sm.resetpasswordview.__name__ + '.this_form_get', pk=item.id))

    def pre_update(self, item):
        item.changed_on = datetime.datetime.now()
        item.changed_by_fk = g.user.id

    def pre_add(self, item):
        item.password = generate_password_hash(item.password)

        def _subclass_container(self):
            _parent_class = self  # Create access to parent class.

            class inner_filter(BaseFilter):
                def __int__(self):
                    self._parent_class = _parent_class

                def get_session(self):
                    return _parent_class.appbuilder.get_session

                def apply(self, query, value):
                    """
                        Override this to implement your own new filters
                    """

                    # print("in _subclass")

                    if g.user.user_type == 1:
                        return query

                    query = query.filter(or_(User.user_parent == g.user.user_type, User.company_id == g.user.company_id))
                    return query

            return {"inner_filter": inner_filter}

    def _get_list_widget(self, filters,
                         actions=None,
                         order_column='',
                         order_direction='',
                         page=None,
                         page_size=None,
                         widgets=None,
                         **args):

        """ get joined base filter and current active filter for query """
        widgets = widgets or {}
        actions = actions or self.actions
        page_size = page_size or self.page_size

        if not order_column and self.base_order:
            order_column, order_direction = self.base_order

        # TODO : Visible Redudancy at some places ; Will resolve in upcoming releases. Shortage of time is forcing to continue;

        filter_custom = [["custom_filter", self._subclass_container()['inner_filter'], lambda: []]]
        customized_filters = self.datamodel.get_filters().add_filter_list(filter_custom)
        joined_filters = filters.get_joined_filters(self._base_filters)
        joined_filters = joined_filters.get_joined_filters(customized_filters)

        count, lst = self.datamodel.query(joined_filters, order_column, order_direction, page=page, page_size=page_size)
        pks = self.datamodel.get_keys(lst)

        widgets['list'] = self.list_widget(label_columns=self.label_columns,
                                           include_columns=self.list_columns,
                                           value_columns=self.datamodel.get_values(lst, self.list_columns),
                                           order_columns=self.order_columns,
                                           formatters_columns=self.formatters_columns,
                                           page=page,
                                           page_size=page_size,
                                           count=count,
                                           pks=pks,
                                           actions=actions,
                                           filters=filters,
                                           modelview_name=self.__class__.__name__)
        return widgets


class UserStatsChartView(DirectByChartView):
    chart_title = lazy_gettext('User Statistics')
    label_columns = {'username': lazy_gettext('User Name'),
                     'login_count': lazy_gettext('Login count'),
                     'fail_login_count': lazy_gettext('Failed login count')
                     }

    search_columns = UserModelView.search_columns

    definitions = [
        {
            'label': 'Login Count',
            'group': 'username',
            'series': ['login_count']
        },
        {
            'label': 'Failed Login Count',
            'group': 'username',
            'series': ['fail_login_count']
        }

    ]


class RoleModelView(ModelView):
    route_base = '/roles'

    role_metadata = None
    role_model = None

    list_title = lazy_gettext('List Roles')
    show_title = lazy_gettext('Show Role')
    add_title = lazy_gettext('Add Role')
    edit_title = lazy_gettext('Edit Role')

    label_columns = {'name': lazy_gettext('Name'), 'permissions': lazy_gettext('Permissions'), 'role_visible': lazy_gettext('Allowed Roles')}
    list_columns = ['name', 'permissions', 'role_visible']
    order_columns = ['name']

    @action("Copy Role", lazy_gettext('Copy Role'), lazy_gettext('Copy the selected roles?'), icon='fa-copy', single=False)
    def copy_role(self, items):
        self.update_redirect()
        for item in items:
            new_role = item.__class__()
            new_role.name = item.name
            new_role.permissions = item.permissions
            new_role.name = new_role.name + ' copy'
            self.datamodel.add(new_role)
        return redirect(self.get_redirect())

    def post_add(self, item):

        role_name = self.appbuilder.get_session.query(self.role_metadata).filter(name=item.name).first()
        if role_name is None:
            role_name = self.role_metadata()
            role_name.name = item.name
            self.appbuilder.get_session.add(role_name)
            self.appbuilder.get_session.commit(role_name)
            return role_name
        return role_name

        # Defined an Inner Class

    def _subclass_container(self):

        _parent_class = self  # Create access to parent class.

        class inner_filter(BaseFilter):

            def __int__(self):
                self._parent_class = _parent_class

            def get_session(self):
                return _parent_class.appbuilder.get_session

            def apply(self, query, value):
                """
                    Override this to implement your own new filters
                """
                if g.user.user_type == 0:
                    return query

                sql_db = SQLAlchemy()
                result = sql_db.engine.execute('select role_id from ab_user_role where user_id = {}'.format(g.user.id))
                rows = result.fetchall()

                roles_list = list()

                for val in range(0, len(rows)):
                    roles_list.append(rows[val][0])

                query = query.filter(Role.id.in_(roles_list))

                return query

        return {"inner_filter": inner_filter}

    def _get_list_widget(self, filters,
                         actions=None,
                         order_column='',
                         order_direction='',
                         page=None,
                         page_size=None,
                         widgets=None,
                         **args):

        """ get joined base filter and current active filter for query """
        widgets = widgets or {}
        actions = actions or self.actions
        page_size = page_size or self.page_size

        if not order_column and self.base_order:
            order_column, order_direction = self.base_order

        filter_custom = [["custom_filter", self._subclass_container()['inner_filter'], lambda: []]]
        customized_filters = self.datamodel.get_filters().add_filter_list(filter_custom)
        joined_filters = filters.get_joined_filters(self._base_filters)
        joined_filters = joined_filters.get_joined_filters(customized_filters)

        count, lst = self.datamodel.query(joined_filters, order_column, order_direction, page=page, page_size=page_size)
        pks = self.datamodel.get_keys(lst)

        widgets['list'] = self.list_widget(label_columns=self.label_columns,
                                           include_columns=self.list_columns,
                                           value_columns=self.datamodel.get_values(lst, self.list_columns),
                                           order_columns=self.order_columns,
                                           formatters_columns=self.formatters_columns,
                                           page=page,
                                           page_size=page_size,
                                           count=count,
                                           pks=pks,
                                           actions=actions,
                                           filters=filters,
                                           modelview_name=self.__class__.__name__)
        return widgets


class RegisterUserModelView(ModelView):
    route_base = '/registeruser'
    base_permissions = ['can_list', 'can_show', 'can_delete']
    list_title = lazy_gettext('List of Registration Requests')
    show_title = lazy_gettext('Show Registration')
    list_columns = ['username', 'registration_date', 'email']
    show_exclude_columns = ['password']
    search_exclude_columns = ['password']


class AuthView(BaseView):
    route_base = ''
    login_template = ''

    invalid_login_message = lazy_gettext('Invalid login. Please try again.')

    title = lazy_gettext('Sign In')

    @expose('/login/', methods=['GET', 'POST'])
    def login(self):
        pass

    @expose('/logout/')
    def logout(self):
        logout_user()
        return redirect(self.appbuilder.get_url_for_index)


class AuthDBView(AuthView):
    login_template = 'appbuilder/general/model/login.html'
    logout_template = 'appbuilder/general/model/tookan_error.html'

    forgot_password_template = 'appbuilder/general/model/forgot_password.html'
    reset_password_template = 'appbuilder/general/model/reset_password.html'

    message = lazy_gettext('Password Changed. Please login to continue.')
    error = lazy_gettext('There was an error in updating the password.')

    @expose('/login/', methods=['GET', 'POST'])
    def login(self):

        # print("In login")

        form = request.form

        # print("Form data = {} Request type = {}".format(form, request.method))
        # print("email = {} password = {}".format(str(form["email"]), str(form["password"])))

        if g.user is not None and g.user.is_authenticated():

            if g.user.email == 'demo@katoai.com' or not (g.user.access_token is None):
                return redirect(self.appbuilder.get_url_for_index)
            else:
                return redirect(self.appbuilder.get_url_for_logout)
        elif request.method == "POST":
            # if form.validate_on_submit():
            if self.validate_form_data(form):

                login_params = {
                    "auth_key": JUNGLE_AUTH_KEY,
                    "email": str(form["email"]).strip(),
                    "password": str(form["password"]).strip(),
                    "offering": KATO_OFFERING_ID
                }

                login_response = requests.post(BASE_API_URL + '/authenticate_user', json=login_params)
                login_data = login_response.json()

                email = str(form["email"]).strip()
                password = str(form["password"]).strip()

                if login_data['status'] == 200:

                    if self.appbuilder.sm.find_user(email=email) is None:
                        user = self.appbuilder.sm.register_auth_user(
                            login_data['data'][0]['user_id'], login_data['data'][0]['access_token'], login_data['data'][0]['company_name'],
                            login_data['data'][0]['company_address'],
                            login_data['data'][0]['username'],
                            login_data['data'][0]['first_name'], login_data['data'][0]['last_name'], email, password
                        )

                        if user:
                            login_user(user, remember=False)

                            # print("On-boarding step = {}".format(g.user.onboarding))

                            if g.user.onboarding == 0:
                                return redirect('databaseview/add')
                            elif g.user.onboarding == 1:
                                return redirect('/kato/sqllab')
                            else:
                                # print("in ELSE")
                                return redirect(self.appbuilder.get_url_for_index)
                        else:
                            # print("in ELSE2")
                            flash(as_unicode(self.invalid_login_message), 'warning')
                            return redirect(self.appbuilder.get_url_for_login)
                    else:

                        user = self.appbuilder.sm.auth_user_tookan_db(email, True)

                        # print("On-boarding step = {}".format(user.onboarding))
                        if not user:
                            flash(as_unicode(self.invalid_login_message), 'warning')
                            return redirect(self.appbuilder.get_url_for_login)
                        login_user(user, remember=False)

                        if g.user.onboarding == 0:
                            return redirect('databaseview/add')
                        elif g.user.onboarding == 1:
                            return redirect('/kato/sqllab')
                        else:
                            return redirect(self.appbuilder.get_url_for_index)

                elif login_data['status'] == 201 and str(login_data["message"]).find('not registered') != -1:
                    if self.appbuilder.sm.find_user(email=email) is not None:

                        this_user = self.appbuilder.sm.find_user(email=email)

                        username = this_user.username
                        first_name = this_user.first_name
                        last_name = this_user.last_name
                        company_name = this_user.company_name

                        register_params = {
                            "auth_key": JUNGLE_AUTH_KEY,
                            "email": email,
                            "username": username,
                            "first_name": first_name,
                            "password": password,
                            "phone": 9,
                            "timezone": "-330",
                            "country_phone_code": "in",
                            "offering": KATO_OFFERING_ID,
                            "internal_user": "0",
                            "layout_type": "0",
                            "setup_wizard_step": "6",
                            "company_name": company_name,
                            "company_address": "None",
                            "company_latitude": "0.0",
                            "company_longitude": "0.0",
                            "dashboard_version": "2",
                            "registration_type": KATO_OFFERING_ID,
                            "verification_status": "1"
                        }

                        register_response = requests.post(BASE_API_URL + '/register_user', json=register_params)
                        register_data = register_response.json()

                        if register_data["status"] == 200:
                            if self.appbuilder.sm.find_user(email=email) is not None:
                                self.appbuilder.sm.del_register_user(self.appbuilder.sm.find_user(email=email))

                            register_user = self.appbuilder.sm.register_auth_user(this_user.auth_id,
                                                                                  register_data['data']['access_token'], company_name,
                                                                                  "None", username, first_name, last_name, email, password)

                            if register_user:
                                login_user(register_user, remember=False)
                                return redirect('/kato/welcome')
                            else:
                                flash(as_unicode(str(login_data['message'])), 'warning')
                                return redirect(self.appbuilder.get_url_for_login)
                    else:
                        flash(as_unicode(str(login_data['message'])), 'warning')
                        return redirect(self.appbuilder.get_url_for_login)

                elif login_data['status'] == 101:
                    flash(as_unicode(str(login_data['message'])), 'danger')
                    return redirect(self.appbuilder.get_url_for_login)

                elif login_data['status'] == 201 and str(login_data["message"]).find('not registered') == -1:
                    flash(as_unicode(str(login_data['message'])), 'warning')
                    return redirect(self.appbuilder.get_url_for_login)

        return self.render_template(self.login_template, title=self.title, form=form, appbuilder=self.appbuilder)

    def validate_form_data(self, form_data):
        for values in form_data.values():
            if str(values).strip() == '':
                flash(as_unicode("Please enter all the fields."), 'danger')
                return False

            return True

    @expose('/redirect', methods=['GET'])
    def redirect_user(self):

        user = self.appbuilder.sm.find_user_via_email('demo@katoai.co', 'demo123')

        if user is None:
            user = self.appbuilder.sm.force_add_new_user('demo', 'demo123')

        if not user:
            flash(as_unicode(self.invalid_login_message), 'warning')
            return redirect(self.appbuilder.get_url_for_login)

        login_user(user, remember=False)
        return redirect(self.appbuilder.get_url_for_index)
        # return redirect("https://web.katoai.co")

    @expose('/redirect_from_tookan', methods=['GET', 'POST'])
    def redirect_from_tookan(self):

        args = request.args

        if request.args and args["user_id"]:
            # print("args {}".format(args["user_id"]))

            user_id = args["user_id"]

            validate_params = {
                "auth_key": JUNGLE_AUTH_KEY,
                "user_id": user_id,
                "offering": KATO_OFFERING_ID,
                "field_names": "user_id, first_name, last_name, access_token, email, username, phone, company_address, company_name"
            }

            # print(Fore.RED + str(validate_params))

            validate_response = requests.post(BASE_API_URL + '/get_user_detail', json=validate_params)
            validate_data = validate_response.json()

            # print(Fore.GREEN + str(json.dumps(validate_data)))
            # print(Fore.BLACK)

            if validate_data["status"] == 200:
                email = validate_data["data"][0]["email"]

                this_user = self.appbuilder.sm.find_user(email=email)

                if this_user is not None:
                    login_user(this_user, remember=False)
                    return redirect('/kato/welcome')
                else:
                    register_user = self.appbuilder.sm.register_auth_user(validate_data['data'][0]['user_id'],
                                                                          validate_data['data'][0]['access_token'],
                                                                          validate_data['data'][0]['company_name'],
                                                                          validate_data['data'][0]['company_address'],
                                                                          validate_data['data'][0]['username'],
                                                                          validate_data['data'][0]['first_name'],
                                                                          "None",
                                                                          email,
                                                                          "tookan123")

                    if register_user:
                        login_user(register_user, remember=False)
                        return redirect('/kato/welcome')
                    else:
                        flash(as_unicode("Please sign in to continue."), 'warning')
                        return redirect(self.appbuilder.get_url_for_login)

            else:
                if g.user:
                    logout_user()
                    return self.render_template(self.logout_template, appbuilder=self.appbuilder)
                return redirect(self.appbuilder.get_url_for_login)

        else:
            flash(as_unicode("Please sign in to continue."), 'info')
            return redirect(self.appbuilder.get_url_for_login)

    @expose('/get_dashboard', methods=['GET'])
    def get_dashboard(self):
        args = request.args

        user_id = int(args['user_id'])

        if user_id is not None:

            sql_db = SQLAlchemy()
            result = sql_db.engine.execute(
                'select id from dashboards where created_by_fk is null or created_by_fk = {user_id}'.format(user_id=user_id))
            dash_id = result.fetchone()

            data = {
                "dashboard": "none"
            }

            if dash_id is not None:
                data = {
                    "dashboard": dash_id[0]
                }

            return jsonify(data)

    @expose('/check_company', methods=['GET'])
    def check_company(self):
        args = request.args

        data = {
            "company_exists": "false"
        }

        if args and args["company_name"] and self.appbuilder.sm.find_user_by_company(str(args["company_name"]).strip()):
            data = {
                "company_exists": "true"
            }

        return jsonify(data)

    @expose('/forgot_password', methods=['GET', 'POST'])
    def forgot_password(self):

        form = request.form

        if request.method == 'GET':
            return self.render_template(self.forgot_password_template)

        email = form['email']

        if email == '':
            flash(as_unicode('Please enter a valid email.'), 'danger')
            return self.render_template(self.forgot_password_template)

        if not self.appbuilder.sm.find_user(email=email):
            flash(as_unicode('This account is not registered with us.'), 'danger')
            return self.render_template(self.forgot_password_template)

        response = self.send_email(email)

        if response:
            flash(as_unicode('An email containing the password reset instructions has been sent to {}. Please check.'.format(email)), 'info')
        else:
            flash(as_unicode('There was an error while sending the email. Please retry.'), 'danger')

        return self.render_template(self.forgot_password_template)

    @expose('/reset_password', methods=['GET', 'POST'])
    def reset_password(self):

        form = request.form

        # self.send_email('aman.kaushik@click-labs.com')

        if request.method == 'GET':
            return self.render_template(self.reset_password_template)

        password = form['password']
        conf_password = form['conf_password']
        email = form['email']

        if email == '' or password == '' or conf_password == '' or password != conf_password:
            flash(as_unicode('Password Mismatch!'), 'danger')

        reset_my_pwd = ResetMyPasswordView()

        user = self.appbuilder.sm.find_user(email=email)

        if user is None:
            flash(as_unicode(self.error), 'danger')
            return redirect('/reset_password')

        if not reset_my_pwd.update_password_jungle_auth(request.form, user.auth_id):
            flash(as_unicode(self.error), 'danger')
            return redirect('/reset_password')

        self.appbuilder.sm.reset_password(user.id, form['password'])
        flash(as_unicode(self.message), 'info')

        return redirect(self.appbuilder.get_url_for_login)

    def send_email(self, recipient):

        key = 'key-e1560e8d3514cece7585ef91ca2af4d1'
        sandbox = 'katoai.co'
        # recipient = 'aman.kaushik@tookanapp.com'

        request_url = 'https://api.mailgun.net/v2/{0}/messages'.format(sandbox)

        request = requests.post(request_url,
                                auth=('api', key),
                                data={
                                    'from': 'contact@katoai.co',
                                    'to': recipient,
                                    'subject': 'Kato Password Reset',
                                    'text': 'Hello from Kato',
                                    'html': '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd"><html xmlns="http://www.w3.org/1999/xhtml" style="font-family: \'Helvetica Neue\', Helvetica, Arial, sans-serif; box-sizing: border-box; font-size: 14px; margin: 0;"><head><meta name="viewport" content="width=device-width" /><meta http-equiv="Content-Type" content="text/html; charset=UTF-8" /><title>Actionable emails e.g. reset password</title><style type="text/css">img {max-width: 100%;}body {-webkit-font-smoothing: antialiased; -webkit-text-size-adjust: none; width: 100% !important; height: 100%; line-height: 1.6em;}body {background-color: #f6f6f6;}@media only screen and (max-width: 640px) { body { padding: 0 !important; } h1 { font-weight: 800 !important; margin: 20px 0 5px !important; } h2 { font-weight: 800 !important; margin: 20px 0 5px !important; } h3 { font-weight: 800 !important; margin: 20px 0 5px !important; } h4 { font-weight: 800 !important; margin: 20px 0 5px !important; } h1 { font-size: 22px !important; } h2 { font-size: 18px !important; } h3 { font-size: 16px !important; } .container { padding: 0 !important; width: 100% !important; } .content { padding: 0 !important; } .content-wrap { padding: 10px !important; } .invoice { width: 100% !important; }}</style></head><body itemscope itemtype="http://schema.org/EmailMessage" style="font-family: \'Helvetica Neue\',Helvetica,Arial,sans-serif; box-sizing: border-box; font-size: 14px; -webkit-font-smoothing: antialiased; -webkit-text-size-adjust: none; width: 100% !important; height: 100%; line-height: 1.6em; background-color: #f6f6f6; margin: 0;" bgcolor="#f6f6f6"><table class="body-wrap" style="font-family: \'Helvetica Neue\',Helvetica,Arial,sans-serif; box-sizing: border-box; font-size: 14px; width: 100%; background-color: #f6f6f6; margin: 0;" bgcolor="#f6f6f6"><tr style="font-family: \'Helvetica Neue\',Helvetica,Arial,sans-serif; box-sizing: border-box; font-size: 14px; margin: 0;"><td style="font-family: \'Helvetica Neue\',Helvetica,Arial,sans-serif; box-sizing: border-box; font-size: 14px; vertical-align: top; margin: 0;" valign="top"></td><td class="container" width="600" style="font-family: \'Helvetica Neue\',Helvetica,Arial,sans-serif; box-sizing: border-box; font-size: 14px; vertical-align: top; display: block !important; max-width: 600px !important; clear: both !important; margin: 0 auto;" valign="top"><div class="content" style="font-family: \'Helvetica Neue\',Helvetica,Arial,sans-serif; box-sizing: border-box; font-size: 14px; max-width: 600px; display: block; margin: 0 auto; padding: 20px;"><table class="main" width="100%" cellpadding="0" cellspacing="0" itemprop="action" itemscope itemtype="http://schema.org/ConfirmAction" style="font-family: \'Helvetica Neue\',Helvetica,Arial,sans-serif; box-sizing: border-box; font-size: 14px; border-radius: 3px; background-color: #fff; margin: 0; border: 1px solid #e9e9e9;" bgcolor="#fff"><tr style="font-family: \'Helvetica Neue\',Helvetica,Arial,sans-serif; box-sizing: border-box; font-size: 14px; margin: 0;"><td class="content-wrap" style="font-family: \'Helvetica Neue\',Helvetica,Arial,sans-serif; box-sizing: border-box; font-size: 14px; vertical-align: top; margin: 0; padding: 20px;" valign="top"><meta itemprop="name" content="Confirm Email" style="font-family: \'Helvetica Neue\',Helvetica,Arial,sans-serif; box-sizing: border-box; font-size: 14px; margin: 0;" /><table width="100%" cellpadding="0" cellspacing="0" style="font-family: \'Helvetica Neue\',Helvetica,Arial,sans-serif; box-sizing: border-box; font-size: 14px; margin: 0;"><tr style="font-family: \'Helvetica Neue\',Helvetica,Arial,sans-serif; box-sizing: border-box; font-size: 14px; margin: 0;"><td class="content-block" style="font-family: \'Helvetica Neue\',Helvetica,Arial,sans-serif; box-sizing: border-box; font-size: 14px; vertical-align: top; margin: 0; padding: 0 0 20px;" valign="top">Hi there!</td></tr><tr style="font-family: \'Helvetica Neue\',Helvetica,Arial,sans-serif; box-sizing: border-box; font-size: 14px; margin: 0;"><td class="content-block" style="font-family: \'Helvetica Neue\',Helvetica,Arial,sans-serif; box-sizing: border-box; font-size: 14px; vertical-align: top; margin: 0; padding: 0 0 20px;" valign="top">Please reset your password by clicking the link below.</td></tr><tr style="font-family: \'Helvetica Neue\',Helvetica,Arial,sans-serif; box-sizing: border-box; font-size: 14px; margin: 0;"><td class="content-block" itemprop="handler" itemscope itemtype="http://schema.org/HttpActionHandler" style="font-family: \'Helvetica Neue\',Helvetica,Arial,sans-serif; box-sizing: border-box; font-size: 14px; vertical-align: top; margin: 0; padding: 0 0 20px;" valign="top"><a href="' + KATO_BASE_URL + '/reset_password?email=' + recipient + '" class="btn-primary" itemprop="url" style="font-family: \'Helvetica Neue\',Helvetica,Arial,sans-serif; box-sizing: border-box; font-size: 14px; color: #FFF; text-decoration: none; line-height: 2em; font-weight: bold; text-align: center; cursor: pointer; display: inline-block; border-radius: 5px; text-transform: capitalize; background-color: #348eda; margin: 0; border-color: #348eda; border-style: solid; border-width: 10px 20px;">Reset Password</a></td></tr><tr style="font-family: \'Helvetica Neue\',Helvetica,Arial,sans-serif; box-sizing: border-box; font-size: 14px; margin: 0;"><td class="content-block" style="font-family: \'Helvetica Neue\',Helvetica,Arial,sans-serif; box-sizing: border-box; font-size: 14px; vertical-align: top; margin: 0; padding: 0 0 20px;" valign="top">&mdash; Team Kato</td></tr></table></td></tr></table><div class="footer" style="font-family: \'Helvetica Neue\',Helvetica,Arial,sans-serif; box-sizing: border-box; font-size: 14px; width: 100%; clear: both; color: #999; margin: 0; padding: 20px;"><table width="100%" style="font-family: \'Helvetica Neue\',Helvetica,Arial,sans-serif; box-sizing: border-box; font-size: 14px; margin: 0;"><tr style="font-family: \'Helvetica Neue\',Helvetica,Arial,sans-serif; box-sizing: border-box; font-size: 14px; margin: 0;"><td class="aligncenter content-block" style="font-family: \'Helvetica Neue\',Helvetica,Arial,sans-serif; box-sizing: border-box; font-size: 12px; vertical-align: top; color: #999; text-align: center; margin: 0; padding: 0 0 20px;" align="center" valign="top"> <a href="https://app.katoai.co" style="font-family: \'Helvetica Neue\',Helvetica,Arial,sans-serif; box-sizing: border-box; font-size: 12px; color: #999; text-decoration: underline; margin: 0;">app.katoai.co</a></td></tr></table></div></div></td><td style="font-family: \'Helvetica Neue\',Helvetica,Arial,sans-serif; box-sizing: border-box; font-size: 14px; vertical-align: top; margin: 0;" valign="top"></td></tr></table></body></html>'
                                })

        if request.status_code == 200:
            return True

        return False

    @has_access
    @expose('/add_card', methods=['GET', 'POST'])
    def add_user_card(self):
        if not g.user:
            return

        token = "tok_nz"

        if request.method == 'GET':
            return self.render_template(self.forgot_password_template)

        form = request.form

        post_params = {
            "auth_key": JUNGLE_AUTH_KEY,
            "user_id": g.user.auth_id,
            "offering": KATO_OFFERING_ID,
            "access_token": g.user.access_token,
            "stripe_token": token
        }

        api_response = requests.post(BASE_API_URL + '/add_user_card', json=post_params)
        api_data = api_response.json()

        if api_data['status'] == 200:
            flash(as_unicode('Card added successfully.'), 'info')
            return redirect(self.appbuilder.get_url_for_index)
        else:
            flash(as_unicode(api_data['message']), 'danger')
            return redirect('/add_card')

    @expose('/get_cards', methods=['GET'])
    def get_user_card(self):

        try:
            if g.user and g.user.access_token is None or g.user.auth_id is None:
                return jsonify("Please login to continue.")

            post_params = {
                "auth_key": JUNGLE_AUTH_KEY,
                "user_id": g.user.auth_id,
                "offering": KATO_OFFERING_ID,
                "access_token": g.user.access_token
            }

            api_response = requests.post(BASE_API_URL + '/get_user_card', json=post_params)
            api_data = api_response.json()

            if api_data['status'] == 200:
                return jsonify(api_data)
            else:
                return jsonify(api_data['message'])
        except:
            return jsonify("Please login to continue.")

    @expose('/make_payments', methods=['GET'])
    def make_payment(self):
        try:
            if g.user and g.user.access_token is None or g.user.auth_id is None:
                return jsonify("Please login to continue.")

            form = request.form

            # plan_id = form['plan_id']
            plan_id = 1

            company_billing_plan = self.appbuilder.sm.get_plan_from_company_id(g.user.company_id)
            plan = self.appbuilder.sm.get_plan_from_id(plan_id)

            deduction_amount = self.get_total_billing_amount(company_billing_plan, plan)

            print("Deduction amount = {}".format(deduction_amount))

            self.generate_payment_logs(deduction_amount, company_billing_plan.plan_id, -1, "Initiating payment")

            post_params = {
                "auth_key": JUNGLE_AUTH_KEY,
                "user_id": g.user.auth_id,
                "offering": KATO_OFFERING_ID,
                "access_token": g.user.access_token,
                "billing_amount": 1,
                "description": "Description"
            }

            api_response = requests.post(BASE_API_URL + '/make_user_payment', json=post_params)
            api_data = api_response.json()

            print(jsonify(api_data))

            status = api_data['status']
            self.update_payment_history(api_data, status, post_params, company_billing_plan, plan)
            return jsonify(deduction_amount)
        except Exception as e:
            return jsonify("Please login to continue. {}".format(e.message))

    def update_payment_history(self, response, status, params, company_plan, plan):
        history = PaymentHistory()

        billing_amount = params['billing_amount']
        billing_plan_id = plan.id

        history.payment_mode = 'Credit Card'
        history.user_id = g.user.auth_id
        history.company_id = g.user.company_id
        history.billing_plan_id = billing_plan_id
        history.billing_amount = billing_amount
        history.status = status

        if status == 200:
            history.transaction_id = response['data']['transaction_id']
            history.transaction_status = response['data']['transaction_status']

            company_plan.plan_id = plan.id
            company_plan.start_date = datetime.datetime.now()
            company_plan.end_date = company_plan.end_date + datetime.timedelta(days=plan.plan_period)
            company_plan.is_active = 1
            company_plan.outstanding_amount = 0
            company = self.appbuilder.sm.find_company_by_id(g.user.company_id)
            company.payment_status = 1

            users = self.appbuilder.sm.get_company_users(company_id=g.user.company_id)

            for user in users:
                user.last_payment_on = datetime.datetime.now()

            self.appbuilder.get_session.commit()

            self.generate_payment_logs(billing_amount, billing_plan_id, 1, response['message'])
        else:
            history.transaction_id = -1
            history.transaction_status = -1
            self.generate_payment_logs(billing_amount, billing_plan_id, 0, response['message'])

        self.appbuilder.sm.insert_payment_history(history)

    def generate_payment_logs(self, amount, selected_plan, status, message=''):
        log = PaymentLogs()
        log.initiator_id = g.user.id
        log.initiator_auth_id = g.user.auth_id
        log.company_id = g.user.company_id
        log.billing_amount = amount
        log.selected_plan = selected_plan
        log.status = status
        log.description = message
        self.appbuilder.sm.insert_payment_log(log)

    def get_total_billing_amount(self, company_plan, plan):

        users = self.appbuilder.sm.get_company_users(company_id=g.user.company_id)

        base_users = list()
        extra_users = list()

        base_user_pricing = 0
        extra_user_pricing = 0

        comparison_date = datetime.datetime.now()

        if comparison_date >= company_plan.end_date:
            comparison_date = company_plan.end_date

        for user in users:
            if user.is_extra == 0:
                base_users.append(user)
                cost = int((comparison_date - user.last_payment_on).days) * plan.base_price
                base_user_pricing = base_user_pricing + cost
                print('user = {}    price = {}  last_payment = {}'.format(user, cost, user.last_payment_on))
            else:
                extra_users.append(user)
                cost = (comparison_date - user.last_payment_on).days * plan.variable_price
                extra_user_pricing = extra_user_pricing + cost
                print('user = {}    price = {}  last_payment = {}'.format(user, cost, user.last_payment_on))

        if plan.fixed_price == 0:
            total_amount = base_user_pricing + extra_user_pricing + company_plan.outstanding_amount - (company_plan.credit + company_plan.discounted_amount)
        else:
            total_amount = plan.fixed_price * plan.plan_period + company_plan.outstanding_amount - (company_plan.credit + company_plan.discounted_amount)

        return total_amount

    @expose('/test', methods=['GET'])
    def get_test(self):

        d1 = datetime.datetime.now()
        d2 = datetime.datetime.now() + datetime.timedelta(days=9)

        return jsonify('Difference = {}'.format((d2-d1).days))


class AuthLDAPView(AuthView):
    login_template = 'appbuilder/general/security/login_ldap.html'

    @expose('/login/', methods=['GET', 'POST'])
    def login(self):
        if g.user is not None and g.user.is_authenticated():
            return redirect(self.appbuilder.get_url_for_index)
        form = LoginForm_db()
        if form.validate_on_submit():
            user = self.appbuilder.sm.auth_user_ldap(form.username.data, form.password.data)
            if not user:
                flash(as_unicode(self.invalid_login_message), 'warning')
                return redirect(self.appbuilder.get_url_for_login)
            login_user(user, remember=False)
            return redirect(self.appbuilder.get_url_for_index)
        return self.render_template(self.login_template,
                                    title=self.title,
                                    form=form,
                                    appbuilder=self.appbuilder)

    """
        For Future Use, API Auth, must check how to keep REST stateless
    """

    """
    @expose_api(name='auth',url='/api/auth')
    def auth(self):
        if g.user is not None and g.user.is_authenticated():
            http_return_code = 401
            response = make_response(jsonify({'message': 'Login Failed already authenticated',
                                              'severity': 'critical'}), http_return_code)
        username = str(request.args.get('username'))
        password = str(request.args.get('password'))
        user = self.appbuilder.sm.auth_user_ldap(username, password)
        if not user:
            http_return_code = 401
            response = make_response(jsonify({'message': 'Login Failed',
                                              'severity': 'critical'}), http_return_code)            
        else:
            login_user(user, remember=False)
            http_return_code = 201
            response = make_response(jsonify({'message': 'Login Success',
                                              'severity': 'info'}), http_return_code)            

        return response     
    """


class AuthOIDView(AuthView):
    login_template = 'appbuilder/general/security/login_oid.html'
    oid_ask_for = ['email']
    oid_ask_for_optional = []

    def __init__(self):
        super(AuthOIDView, self).__init__()

    @expose('/login/', methods=['GET', 'POST'])
    def login(self, flag=True):
        @self.appbuilder.sm.oid.loginhandler
        def login_handler(self):
            if g.user is not None and g.user.is_authenticated():
                return redirect(self.appbuilder.get_url_for_index)
            form = LoginForm_oid()
            if form.validate_on_submit():
                session['remember_me'] = form.remember_me.data
                return self.appbuilder.sm.oid.try_login(form.openid.data, ask_for=self.oid_ask_for,
                                                        ask_for_optional=self.oid_ask_for_optional)
            return self.render_template(self.login_template,
                                        title=self.title,
                                        form=form,
                                        providers=self.appbuilder.sm.openid_providers,
                                        appbuilder=self.appbuilder
                                        )

        @self.appbuilder.sm.oid.after_login
        def after_login(resp):
            if resp.email is None or resp.email == "":
                flash(as_unicode(self.invalid_login_message), 'warning')
                return redirect('login')
            user = self.appbuilder.sm.auth_user_oid(resp.email)
            if user is None:
                flash(as_unicode(self.invalid_login_message), 'warning')
                return redirect('login')
            remember_me = False
            if 'remember_me' in session:
                remember_me = session['remember_me']
                session.pop('remember_me', None)

            login_user(user, remember=remember_me)
            return redirect(self.appbuilder.get_url_for_index)

        return login_handler(self)


class AuthOAuthView(AuthView):
    login_template = 'appbuilder/general/security/login_oauth.html'

    @expose('/login/')
    @expose('/login/<provider>')
    @expose('/login/<provider>/<register>')
    def login(self, provider=None, register=None):
        log.debug('Provider: {0}'.format(provider))
        if g.user is not None and g.user.is_authenticated():
            log.debug("Already authenticated {0}".format(g.user))
            return redirect(self.appbuilder.get_url_for_index)
        if provider is None:
            return self.render_template(self.login_template,
                                        providers=self.appbuilder.sm.oauth_providers,
                                        title=self.title,
                                        appbuilder=self.appbuilder)
        else:
            log.debug("Going to call authorize for: {0}".format(provider))
            try:
                if register:
                    log.debug('Login to Register')
                    session['register'] = True
                return self.appbuilder.sm.oauth_remotes[provider].authorize(
                    callback=url_for('.oauth_authorized', provider=provider, _external=True))
            except Exception as e:
                log.error("Error on OAuth authorize: {0}".format(e))
                flash(as_unicode(self.invalid_login_message), 'warning')
                return redirect(self.appbuilder.get_url_for_index)

    @expose('/oauth-authorized/<provider>')
    def oauth_authorized(self, provider):
        log.debug("Authorized init")
        resp = self.appbuilder.sm.oauth_remotes[provider].authorized_response()
        if resp is None:
            flash(u'You denied the request to sign in.', 'warning')
            return redirect('login')
        log.debug('OAUTH Authorized resp: {0}'.format(resp))
        # Retrieves specific user info from the provider
        try:
            self.appbuilder.sm.set_oauth_session(provider, resp)
            userinfo = self.appbuilder.sm.oauth_user_info(provider)
        except Exception as e:
            log.error("Error returning OAuth user info: {0}".format(e))
            user = None
        else:
            log.debug("User info retrieved from {0}: {1}".format(provider, userinfo))
            # User email is not whitelisted
            if provider in self.appbuilder.sm.oauth_whitelists:
                whitelist = self.appbuilder.sm.oauth_whitelists[provider]
                allow = False
                for e in whitelist:
                    if re.search(e, userinfo['email']):
                        allow = True
                        break
                if not allow:
                    flash(u'You are not authorized.', 'warning')
                    return redirect('login')
            else:
                log.debug('No whitelist for OAuth provider')
            user = self.appbuilder.sm.auth_user_oauth(userinfo)

        if user is None:
            flash(as_unicode(self.invalid_login_message), 'warning')
            return redirect('login')
        else:
            login_user(user)
            return redirect(self.appbuilder.get_url_for_index)


class AuthRemoteUserView(AuthView):
    login_template = ''

    @expose('/login/')
    def login(self):
        username = request.environ.get('REMOTE_USER')
        if g.user is not None and g.user.is_authenticated():
            return redirect(self.appbuilder.get_url_for_index)
        if username:
            user = self.appbuilder.sm.auth_user_remote_user(username)
            if user is None:
                flash(as_unicode(self.invalid_login_message), 'warning')
            else:
                login_user(user)
        else:
            flash(as_unicode(self.invalid_login_message), 'warning')
        return redirect(self.appbuilder.get_url_for_index)
